# Custom Warp script by Defiant <mail@defiant.homedns.org>

# Settings:
# =========

# Max Acceleration
WarpAccelFac = 5
# Max Speed
MaxWarpFacMul = 6

ManoevreBlock = 4


# ==============================

# Imports
import App
import MissionLib
import Bridge.BridgeUtils
import Lib.Ambiguity
import QuickBattle.QuickBattle

# Vars
pBridge = App.g_kSetManager.GetSet('bridge')
BridgeType = App.BridgeSet_Cast(pBridge).GetConfig()
g_pKiska = App.CharacterClass_GetObject(pBridge, 'Helm')
pKiskaMenu = Bridge.BridgeUtils.GetBridgeMenu('Helm')
CUSTOM_WARP = App.Mission_GetNextEventType()
CustomWarpStatus = 0
PhaserWasOn = 0
PulseWasOn = 0
pShipPre = None
SHIP_STOPPED = App.Mission_GetNextEventType()
pShipMaxSpeed = None
MaxWarpFac = None
	
def CustomWarp(pObject, pEvent):
	global CustomWarpStatus, PhaserWasOn, PulseWasOn, pShipPre, WarpAccelFac, MaxWarpFac, ManoevreBlock, pButton, pShipMaxSpeed, MaxWarpFacMul
	
	pShip = App.Game_GetCurrentPlayer()
	
	if (pShipPre == None):
		pShipPre = str(pShip)
	elif(pShipPre != str(pShip)):
		print("We switched the Ship, make sure Warp is not on")
		CustomWarpStatus = 0
		pShipPre = None
	
	if (pShip.GetWarpEngineSubsystem() == None):
		print("This ship does not have any Warp drives. Return")
		return
	
	if ( CustomWarpStatus == 0 ):
		if (pShip.GetImpulseEngineSubsystem() != None):
			# Status
			print (pShip.GetImpulseEngineSubsystem().GetMaxSpeed())
			print (pShip.GetImpulseEngineSubsystem().GetMaxAccel())
			print (pShip.GetImpulseEngineSubsystem().GetMaxAngularVelocity())
			print (pShip.GetImpulseEngineSubsystem().GetMaxAngularAccel())
			
			# Count the Ship Size into the MaxSpeed
			MaxWarpFac = (MaxWarpFacMul + MaxWarpFacMul + pShip.GetHull().GetRadius()) / 2
			
			# Speed ++
			pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxSpeed(pShip.GetImpulseEngineSubsystem().GetMaxSpeed() * MaxWarpFac)
			pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxAccel(pShip.GetImpulseEngineSubsystem().GetMaxAccel() * WarpAccelFac)
			# less Manoeuvre
			pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxAngularVelocity(pShip.GetImpulseEngineSubsystem().GetMaxAngularVelocity() / ManoevreBlock)
			pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxAngularAccel(pShip.GetImpulseEngineSubsystem().GetMaxAngularAccel() / ManoevreBlock)
			# Go
			pShip.SetImpulse(9, App.TGPoint3_GetModelForward(), App.PhysicsObjectClass.DIRECTION_MODEL_SPACE)
			
		# Deactivate non-torpedo Weapons
		if (pShip.GetPhaserSystem() != None):
			if (pShip.GetPhaserSystem().IsOn() == 1):
				pShip.GetPhaserSystem().Turn(0)
				PhaserWasOn = 1
		if (pShip.GetPulseWeaponSystem() != None):
			if (pShip.GetPulseWeaponSystem().IsOn() == 1):
				pShip.GetPulseWeaponSystem().Turn(0)
				PulseWasOn = 1
		# Custom Warp is on
		CustomWarpStatus = 1
	else:
		# All Stop
		pShipMaxSpeed = pShip.GetImpulseEngineSubsystem().GetMaxSpeed() / MaxWarpFac
		if (pShip.GetImpulseEngineSubsystem() != None):
			pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxSpeed(0)

		# Disable Button so Player can't Warp again before the Ship is not out of Warp
		pButton.SetDisabled()
			
		StopShipTimer(5) #Give the Ship 5 Seconds to go out of Warp
			
			
def StopShipTimer(iCountdown):
		# Create an event - it's a thing that will call this function
		pTimerEvent = App.TGEvent_Create()
		pTimerEvent.SetEventType(SHIP_STOPPED)
		pTimerEvent.SetDestination(App.TopWindow_GetTopWindow())
					
		# Create a timer - it's a thing that will wait for a given time,then do something
		pTimer = App.TGTimer_Create()
		pTimer.SetTimerStart(App.g_kUtopiaModule.GetGameTime() + iCountdown)
		pTimer.SetDelay(0)
		pTimer.SetDuration(0)
		pTimer.SetEvent(pTimerEvent)
		App.g_kTimerManager.AddTimer(pTimer)
		
		
def ShipIsOutOfWarp(pObject, pEvent):
	global PhaserWasOn, PulseWasOn, WarpAccelFac, MaxWarpFac, ManoevreBlock, pButton, pShipMaxSpeed, CustomWarpStatus
	
	pShip = App.Game_GetCurrentPlayer()
	
	# Reset Engines
	if (pShip.GetImpulseEngineSubsystem() != None):
		pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxSpeed(pShipMaxSpeed)
		pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxAccel(pShip.GetImpulseEngineSubsystem().GetMaxAccel() / WarpAccelFac)
		pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxAngularVelocity(pShip.GetImpulseEngineSubsystem().GetMaxAngularVelocity() * ManoevreBlock)
		pShip.GetImpulseEngineSubsystem().GetProperty().SetMaxAngularAccel(pShip.GetImpulseEngineSubsystem().GetMaxAngularAccel() * ManoevreBlock)
	# Weapons
	if (PhaserWasOn == 1 ):
		pShip.GetPhaserSystem().Turn(1)
		PhaserWasOn = 0
	if (PulseWasOn == 1 ):
		pShip.GetPulseWeaponSystem().Turn(1)
		PulseWasOn = 0
	# Button
	pButton.SetEnabled()

	# We are out of Warp
	CustomWarpStatus = 0


if (pKiskaMenu != None):
	event = Lib.Ambiguity.getEvent(CUSTOM_WARP)
	g_pKiska.AddPythonFuncHandlerForInstance(event, __name__ + ".CustomWarp")
	pButton = QuickBattle.QuickBattle.CreateBridgeMenuButton(App.TGString("In-System Warp"), event, 0, g_pKiska)
	pKiskaMenu.PrependChild(pButton)

App.TopWindow_GetTopWindow().AddPythonFuncHandlerForInstance(SHIP_STOPPED, __name__ + ".ShipIsOutOfWarp")


# Make sure Custom Warp is really off on restart
def Restart():
	global CustomWarpStatus
	CustomWarpStatus = 0
